package dbclase;

import java.io.IOException;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import javax.swing.JOptionPane;

/**
 * Clase para probar el acceso a bases de datos desde Java.
 * Como no es más que un ejercicio, todos los parámetros estan "hardcodeados" como campos estáticos finales.
 *
 * @author gbg
 */
public class DBClase {
    //URL base donde se encuentran los ficheros a procesar.
    private static final String URLDIR="http://gtts.ehu.es/German/Docencia/assets/";
    //Nombres de los ficheros a procesar.
    private static final String[] FILENAMES={"20171201.txt", "20171211.txt", "20171214.txt", "20171215.txt"};
    //Nombre a dar a la base de datos.
    private static final String DATABASE_NAME="actaspv";
    //Nombre de la tabla a incluir en la base de datos.
    private static final String TABLE_NAME="estadisticos";
    //Objeto conexión con la base de datos
    private static Connection con;

    // Con cada arranque se comienza con la base de datos vacia. Para ello se elimina si existe previamente
    // y se vuelve a crear.
    static {
        try {
            //Se establece la conexión, que queda disponible para toda la ejecución (es un campo de la clase) 
            con=DriverManager.getConnection("jdbc:mysql://localhost:3306/","german","chicle");
            //Se genera la base de datos destruyéndola previamente si existe. 
            con.createStatement().executeUpdate("DROP DATABASE IF EXISTS "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("CREATE DATABASE "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("USE "+DATABASE_NAME+";");
            con.createStatement().executeUpdate(
                        "CREATE TABLE "+TABLE_NAME+" ("
                        +"`id` VARCHAR(8) NOT NULL,"
                        +"`numPalabras` INT NOT NULL,"
                        +"`validas` INT NOT NULL,"
                        +"`media` DOUBLE NOT NULL,"
                        +"`desviacionTipica` DOUBLE NOT NULL,"
                        +"PRIMARY KEY (`id`),"
                        +"UNIQUE INDEX `id_UNIQUE` (`id` ASC));"
            );
        } catch (SQLException ex) {
            //En caso de problemas se muestra un mensaje con un cuadro de dialogo y terminamos.
            JOptionPane.showMessageDialog(null,"<html>Lo sentimos. Error fatal:<br/>"+ex.getMessage()+"<br/>Adiós.");
            System.exit(-1);
        }
    }

    /**
     * Pruebas de construcción, escritura y lectura de base de datos
     * 
     * @param args no hay argumentos de entrada al programa. Es una práctica "hardcodeada"
     */
    public static void main(String[] args) {
        try {
            // PARTE PRIMERA: Procesamiento de datos y envio de resultados a la base de datos

            // Preparamos un statement para ir conformando las órdenes de inserción de registros en la base de datos.
            PreparedStatement pstmt = con.prepareStatement(
                    "insert into "+TABLE_NAME+" (id, numPalabras, validas, media, desviacionTipica) values (?, ?, ?, ?, ?)"
            );

            // Recorremos todos los ficheros uno a uno
            for (String filename : FILENAMES) {
                try {
                    // Se procesa el fichero obteniendo un objeto que es un conjunto de objetos de nuestro interés
                    FirstPhoneneSet f=new FirstPhoneneSet(URLDIR+filename);
                    // y con él se cumplimentan los cinco campos de statement que tenemos preparado
                    pstmt.setString(1, filename.substring(0,filename.lastIndexOf('.')));
                    pstmt.setInt(2, f.palabrasTotales());
                    pstmt.setInt(3, f.palabrasValidas());
                    pstmt.setDouble(4,f.media());
                    pstmt.setDouble(5,f.desviacion());
                    // y lo añadimos al "batch" que finalmente irá a la base de datos
                    pstmt.addBatch();
                } catch (IOException ex) {
                    //En caso de problemas se muestra un mensaje con un cuadro de dialogo.
                    //no es la mejor opción porque paraliza la ejecución hasta que se atiende.
                    //TODO estudiar lo que hace la linea que pone Netbeans "de oficio" (que es la solución correcta)
                    JOptionPane.showMessageDialog(null,"<html>Lo sentimos:<br/>"+ex.getMessage());
                }
            }
            // acabados de procesar todos los ficheros, ejecutamos el batch para generar todos los registros.
            pstmt.executeBatch();

            // PARTE SEGUNDA: Consulta a la base de datos para obtener la tabla completa

            // Pedimos la tabla
            ResultSet rs = con.createStatement().executeQuery("select * from "+TABLE_NAME);
            // y del resultado obtenido recogemos los metadatos
            ResultSetMetaData rsmd=rs.getMetaData();
            // de estos sacamos los nombres de las columnas y los mostramos en la salida
            for (int i=1;i<= rsmd.getColumnCount();i++) System.out.print(rsmd.getColumnName(i)+"\t");
            System.out.println();
            // y recorremos las filas mostrando los valores de las 5 columnas (cuyos tipos conocemos de antemano)
            while (rs.next()) {
                System.out.print(rs.getString(1)+"\t");
                System.out.print(rs.getInt(2)+"\t");
                System.out.print(rs.getInt(3)+"\t");
                System.out.print(rs.getDouble(4)+"\t");
                System.out.println(rs.getDouble(5));
            }

        } catch (SQLException ex) {
            //En caso de problemas se muestra un mensaje con un cuadro de dialogo y terminamos.
            JOptionPane.showMessageDialog(null,"<html>Lo sentimos. Error fatal:<br/>"+ex.getMessage()+"<br/>Adiós.");
            System.exit(-1);
        }
    }
}
